#!/usr/bin/python

#This script will calculate what distros need what MythTV releases.
#It makes a couple of assumptions that should be noted if^d^dwhen it breaks

# * Some release of Ubuntu contains the latest stable version of MythTV
#   This can be a development release, but it needs to be somewhere.

# * Standard ubuntu releases get builds for two mythtv versions, the one they
#   shipped with and one more.

# * LTS ubuntu releases get builds for all mythtv versions

# * Only LTS releases have point releases (that's how we calculate them)

# * Master is uploaded to Ubuntu development only, never a regular release

import sys
import os
import subprocess
from launchpadlib.launchpad import Launchpad
cachedir = os.path.join(os.environ['HOME'], '.launchpadlib', 'cache')
launchpad = Launchpad.login_anonymously('MythTV version checker', 'production', cachedir)

GITHUB_URL='git://github.com/MythTV/mythtv.git'

def find_distros():
    distros = []
    ubuntu = launchpad.distributions["ubuntu"]
    archive = ubuntu.main_archive
    series_collection = ubuntu.series_collection

    if 'DEBUG' in os.environ:
        print "# ->Distro information"
    for series in series_collection:
        distro={}
        distro['enabled'] = series.active
        #88mph?
        if 'Future' in series.status:
            continue
        #LTS?
        distro['lts_release'] = False
        for milestone in series.all_milestones:
            if series.version in milestone.name and \
               not 'beta' in milestone.name:
                spl = milestone.name.split(series.version)[1]
                if spl.startswith('.'):
                    distro['lts_release'] = True
                    break
        #Targets for master
        if 'DEBUG' in os.environ:
            print "# ..release: %s. status: %s" % (series.version, series.status)
        distro['development_target'] = 'Active Development' in series.status
        distro['current_release'] = 'Current Stable Release' in series.status
        distro['name'] = series.name
        distro['version'] = series.version
        full_version = archive.getPublishedSources(exact_match=True,source_name="mythtv", distro_series=series)[0].source_package_version
        if '+' in full_version:
            delimitter = '+'
            distro['type'] = 'fixes'
        elif '~' in full_version:
            delimitter = '~'
            distro['type'] = 'master'
        else:
            delimitter = ''
            version = full_version
            distro['type'] = 'fixes'
        if delimitter:
            version = full_version.split(delimitter)[0].rsplit('.', 1)[0]
        #epoch
        if ':' in full_version:
            version = version.split(':')[1]
        try:
            distro['shipped_mythtv_version'] = float(version)
        except ValueError:
            distro['shipped_mythtv_version'] = -1
        distros.append(distro)

    #launchpad gives us new->old, we want old->new for publishing to PPA
    #distros.reverse()
    return distros

def calculate_fixes_version(distros):
    """Calculates versions assuming the latest version
       in the archive is the latest stable fixes release"""
    fixes_release = float(0)
    #Find latest fixes version and it's our central pivot point
    for distro in distros:
        if distro['shipped_mythtv_version'] > fixes_release and distro['type'] == 'fixes':
            fixes_release = distro['shipped_mythtv_version']
    #check newest upstream version
    call = subprocess.Popen(['git', 'ls-remote', GITHUB_URL, 'refs/heads/fixes/*'], stdout=subprocess.PIPE)
    out,err = call.communicate()
    for line in out.split('\n'):
        if line:
            number = float(line.split('fixes/')[1])
            if number > float(fixes_release):
                fixes_release = number
    return fixes_release

def calculate_point_release(version):
    """Calculates the newest point release for version"""
    call = subprocess.Popen(['git', 'ls-remote', GITHUB_URL, 'refs/tags/v' + str(version) + '.*'], stdout=subprocess.PIPE)
    out,err = call.communicate()
    biggest = 0
    for line in out.split('\n'):
        if line and not '^' in line:
            number = int(line.split('/v' + str(version) + '.')[1])
            if number > biggest:
                biggest = number
    return biggest

def calculate_targets(mythtv_type, distros):
    '''Calculates the releases'''
    targets = ''
    max_lts = 2
    pivot = calculate_fixes_version(distros)
    if mythtv_type == 'fixes':
        if pivot == 29.0:
            releases = (29.0, 0.28)
        else:
            releases = (pivot, round(pivot -1.0, 2))
    elif mythtv_type == 'fixes_old':
        if pivot == 29.0:
            releases = (0.28, 0.27)
        else:
            releases = (round(pivot -1.0, 2), round(pivot -2.0, 2))
    else:
        releases = (pivot, round(pivot +1.0, 2))

    if 'DEBUG' in os.environ:
        print "# ->%s releases: %s" % (mythtv_type, releases)
    for distro in distros:
        #add fixes builds
        if mythtv_type == 'fixes':
            if distro['shipped_mythtv_version'] in releases:
                if targets:
                    if not distro['name'] in targets and distro['enabled']:
                        targets = ' '.join([distro['name'], targets])
                elif distro['enabled']:
                    targets = distro['name']
                if 'DEBUG' in os.environ:
                    print "# ..%s" % distro
        #add master builds
        elif mythtv_type == 'master':
            if distro['development_target'] or distro['current_release'] or \
               distro['shipped_mythtv_version'] in releases:
                if targets:
                    if not distro['name'] in targets and distro['enabled']:
                        targets = ' '.join([distro['name'], targets])
                elif distro['enabled']:
                    targets = distro['name']
                if 'DEBUG' in os.environ:
                    print "# ..%s" % distro
        elif mythtv_type == 'fixes_old':
            if distro['shipped_mythtv_version'] in releases:
                if targets:
                    if not distro['name'] in targets and distro['enabled']:
                        targets = ' '.join([distro['name'], targets])
                elif distro['enabled']:
                    targets = distro['name']
                if 'DEBUG' in os.environ:
                    print "# ..%s" % distro
        #add last 2 LTS releases in if not already there
        if distro['enabled'] and distro['lts_release']:
            if distro['name'] not in targets and max_lts > 0:
                if 'DEBUG' in os.environ:
                    print "# ..%s" % distro
                targets = ' '.join([distro['name'], targets])
            max_lts = max_lts - 1
    return targets

## Load Blacklist
BLACKLIST = {}
f = open('black.list', 'r')
for l in f.readlines():
    if not l.startswith("#"):
        ver, rels = l.strip().split("\t",1)
        BLACKLIST[ver] = rels
f.close()
if 'DEBUG' in os.environ:
    print "# ->Blacklist"
    print "# ..%s" % BLACKLIST

distros = find_distros()

fixes_release = calculate_fixes_version(distros)
master_release = fixes_release + 1
#fixes_old_release = fixes_release - 1  ## This needs to be uncommented for the 30.0 release
fixes_old_release = 0.28  ## This needs to be commented out for the 30.0 release.
fixes_minor_release = calculate_point_release(fixes_release)
fixes_old_minor_release = calculate_point_release(fixes_old_release)

if 'DEBUG' in os.environ:
    print 'Fixes release: %s' % fixes_release
    print 'Master release: %s' % master_release
    print 'Fixes Old release: %s' % fixes_old_release
    print 'Fixes Minor release: %s' % fixes_minor_release
    print 'Fixes Old Minor release: %s' % fixes_old_minor_release


raw_fixes_old_targets = calculate_targets('fixes_old', distros)
raw_fixes_targets = calculate_targets('fixes', distros)
raw_master_targets = calculate_targets('master', distros)

if 'DEBUG' in os.environ:
    print 'Raw Fixes Old Targets: %s' % raw_fixes_old_targets
    print 'Raw Fixes Targets: %s' % raw_fixes_targets
    print 'Raw Master Targets: %s' % raw_master_targets


## Remove blacklisted items
master_targets = ''
for l in raw_master_targets.split():
    if 'DEBUG' in os.environ:
        print "Checking %s %s against BLACKLIST %s for Master" % (str(master_release),l,str(BLACKLIST))
    if not str(master_release) in BLACKLIST:
        if 'DEBUG' in os.environ:
            print "Adding %s to master builds" % l
        master_targets = ' '.join([l, master_targets])
    elif not l in BLACKLIST[str(master_release)]:
        if 'DEBUG' in os.environ:
            print "Adding %s to master builds" % l
        master_targets = ' '.join([l, master_targets])

fixes_targets = ''
for l in raw_fixes_targets.split():
    if 'DEBUG' in os.environ:
        print "Checking %s %s against BLACKLIST %s for Fixes" % (str(fixes_release),l,str(BLACKLIST))
    if not str(fixes_release) in BLACKLIST:
        if 'DEBUG' in os.environ:
            print "Adding %s to fixes builds" % l
        fixes_targets = ' '.join([l, fixes_targets])
    elif not l in BLACKLIST[str(fixes_release)]:
        if 'DEBUG' in os.environ:
            print "Adding %s to fixes builds" % l
        fixes_targets = ' '.join([l, fixes_targets])

fixes_old_targets = ''
for l in raw_fixes_old_targets.split():
    if 'DEBUG' in os.environ:
        print "Checking %s %s against BLACKLIST %s for Old Fixes" % (str(fixes_old_release),l,str(BLACKLIST))
    if not str(fixes_old_release) in BLACKLIST:
        if 'DEBUG' in os.environ:
            print "Adding %s to fixes_old builds" % l
        fixes_old_targets = ' '.join([l, fixes_old_targets])
    elif not l in BLACKLIST[str(fixes_old_release)]:
        if 'DEBUG' in os.environ:
            print "Adding %s to fixes_old builds" % l
        fixes_old_targets = ' '.join([l, fixes_old_targets])


if 'DEBUG' in os.environ:
    print "# ->Actual output"
print "export GIT_MAJOR_RELEASE_FIXES_OLD=%d" % round(fixes_old_release*100,3)
print "export GIT_MINOR_RELEASE_FIXES_OLD=%d" % fixes_old_minor_release
print "export GIT_MAJOR_RELEASE_FIXES=%d" % round(fixes_release,3)
print "export GIT_MINOR_RELEASE_FIXES=%d" % fixes_minor_release
print "export GIT_MAJOR_RELEASE_MASTER=%d" % round(master_release,3)
print "export GIT_MINOR_RELEASE_MASTER=0"
print "export DISTROS_FIXES_OLD='%s'" % fixes_old_targets
print "export DISTROS_FIXES='%s'" % fixes_targets
print "export DISTROS_MASTER='%s'" % master_targets
