#!/usr/bin/env python

from launchpadlib.launchpad import Launchpad
import re
import time
import os
import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

HOMEPATH='/root'

# create a file handler
handler = logging.FileHandler('%s/build_mover.log' % HOMEPATH)
handler.setLevel(logging.DEBUG)

# create a logging format
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)

# add the handlers to the logger
logger.addHandler(handler)

## Login to LP
#cachedir = os.path.join(os.environ['HOME'], '.launchpadlib', 'cache')
#launchpad = Launchpad.login_with('tgm4883', 'production', cachedir)  ## For testing, needs to login with permissions to move/delete packages
launchpad = Launchpad.login_with('pegasus+', 'production',  credentials_file='%s/creds' % HOMEPATH)
#launchpad = Launchpad.login_with('pegasus+', 'production')

## Set team
mbteam = launchpad.people['mythbuntu']

## Set source PPA
#srcppa = mbteam.getPPAByName(name="fixes")  ## For testing
BUILDERPPA = [
#mbteam.getPPAByName(name="0.28"),
mbteam.getPPAByName(name="fixes-building"),
mbteam.getPPAByName(name="master-building"),
mbteam.getPPAByName(name="fixes-old-building"),
]

SrcPPAPublishedSources = []
ToDelete = []

## Print starting timestamp
logger.info("")
logger.info(time.strftime("***** Checking LP for builds to move on %Y-%m-%d at %H:%M:%S *****"))
logger.info("")

def copySource(DISTRO, VERSION, SRCITEM, SRCPPA):
    '''Copy the source package into the new repository'''
    logger.debug("copySource: %s" % VERSION)
    regex='([0-9][0-9])\.([0-9])'
    logger.debug("Try: %s" % regex)
    m = re.search(regex, VERSION)
    destination=m.group(1)
    logger.debug("Destination Group: %s" % destination)
    if destination == '28':
        destination = '0.28'
    elif destination == '29':
        destination='0.29'
    logger.debug("Destination PPA: %s" % destination)
    logger.info("Moving %s for %s to %s " % (VERSION, DISTRO, destination))
    dstppa = mbteam.getPPAByName(name=destination) ## Set destination PPA
    #dstppa = launchpad.me.getPPAByName(name="staging")  ## For testing
    dstppa.syncSource(
        from_archive=SRCPPA,
        include_binaries=True, ## Don't rebuild the binaries, just copy them
        source_name="mythtv",
        to_pocket="release",
        to_series=DISTRO, ## This is the distro to copy it to
        version=VERSION)  ## This is the version to copy from
    logger.debug("Adding %s to TODELETE" % SRCITEM)
    ToDelete.append((SRCITEM, destination, "Moved")) ## Add the package version to delete
    #for i in FinishedBuilds:
    #    logger.debug("Adding %s to TODELETE" % i)
    #    ToDelete.append((i, destination, "Moved"))

def deleteSource(ITEM, LOC, REASON):
    logger.info("Deleting item %s from %s because %s" % (ITEM, LOC, REASON))
    if REASON == "Moved":
        ITEM.requestDeletion(removal_comment="Auto-Moved to "+str(LOC))
    else:
        ITEM.requestDeletion(removal_comment="Build superseded before completion!!!")

logger.debug("BUILDERPPA %s" % BUILDERPPA)

for SRCPPA in BUILDERPPA:
    logger.debug("SRCPPA: %s" % SRCPPA)
    for l in SRCPPA.getPublishedSources(source_name='mythtv', status='Published'):
        logger.debug("Processing %s" % l)
        if not l.status == "Deleted":
            logger.debug("%s not status deleted" % l)
            if l.status == "Superseded":
                logger.debug("%s status is Superseded. Sending related items for deletion" % l)
                #for item in SRCPPA.getPublishedBinaries(binary_name='mythtv-frontend', version=l.source_package_version):
                #    ToDelete.append((item, None, "Superseded"))
                ToDelete.append((l, None, "Superseded"))
            else:
                logger.debug("Verifying %s is ready to be moved" % l)
                FinishedBuilds=[]
                Builds=[]
                for i in l.getBuilds():
                    Builds.append(i)
                for i in Builds:
                    if 'Successfully built' in i.buildstate: ## If it's successful, add it to finished builds list
                        logger.debug("%s has built successfully" % i.arch_tag)
                        FinishedBuilds.append(i.arch_tag)
                    elif 'armhf' in i.arch_tag and 'Failed to build' in i.buildstate: ## only add failed ARMHF builds to finished list (this should be removed once ARM builders are more reliable)
                        FinishedBuilds.append(i.arch_tag)
                    else:
                        logger.debug("%s is currently %s" % (i.arch_tag, i.buildstate))
                pkg, version, tw, distro = l.display_name.split()
                if len(FinishedBuilds) == len(Builds):
                    logger.debug("Adding %s to move list" % l.source_package_version)
                    copySource(distro, l.source_package_version, l, SRCPPA)
                else:
                    logger.info("%s not ready for %s (currently at %s)" % (l.source_package_version, distro, str(len(FinishedBuilds))))
        logger.debug("%s already deleted" % l)

## Delete copied items
logger.debug(ToDelete)
for I, L, R in ToDelete:
    deleteSource(I, L, R)

## Print finishing timestamp
logger.info("")
logger.info(time.strftime("\n***** Finished checking LP for builds to move on %Y-%m-%d at %H:%M:%S *****\n"))
logger.info("")
