#!/usr/bin/python

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid.axislines import Subplot
import pylab
pylab.rcParams['legend.loc'] = 'best'
pylab.rcParams['legend.shadow'] = True
pylab.rcParams['legend.fancybox'] = True

import sys
import os
#import paramiko
import pysftp
import shutil
import datetime
import re

from launchpadlib.launchpad import Launchpad
cachedir = os.path.join(os.environ['HOME'], '.launchpadlib', 'cache')
launchpad = Launchpad.login_anonymously('MythTV version checker', 'production', cachedir)
user = launchpad.people('mythbuntu')

LEGEND_COLORS={'#E95420': 'Supported', '#772953': 'Long Term Support', 'black': 'Obsolete', '#AD7E97': 'Active Development'}

collection = []
supported = {}
mintmap = {}
SUPLTS=[]
IGNORELTS=[]

def load_mint():
    f = open('mint.map', 'r')
    for l in f.readlines():
        u, m = l.strip().split('\t')
        mintmap[m] = u
    f.close()

def valid_ppas():
    """Returns a list of valid PPAs.  Assumes the biggest is last"""
    for ppa in user.ppas:
        try:
            try:
                regex='([0-9])\.([0-9][0-9])'
                m = re.search(regex, ppa.displayname)
                destination=m.group(0)
                collection.append(ppa)
            except:
                regex='([0-9][0-9])\.([0-9])'
                m = re.search(regex, ppa.displayname)
                destination=m.group(0)
                collection.append(ppa)
        except:
            pass
    collection.sort(key = lambda item: item.displayname)

def second_biggest_ppa():
    cur = collection[-2]
    dev = collection[-1]
    return cur.name, dev.name

def write_header():
    print >>output, "MYTHTV_RELEASE\t%s" % second_biggest_ppa()[0]
    print >>output, "TRUNKPASS\tYouMustBeThisTallToRide"
    print >>output, "URL\thttp://people.ubuntu.com/~tgm4883/repos.db"
    print >>output, ""

def iterate_ppas():
    results = {}
    for ppa in collection:
        for series in ppa.distribution.series_collection:
            if series.status == "Future":
                continue
            source = ppa.getPublishedSources(exact_match=True,source_name="mythtv", distro_series=series, status="Published")
            if source:
                for milestone in series.all_milestones:
                    if series.version in milestone.name and \
                       not 'beta' in milestone.name:
                        spl = milestone.name.split(series.version)[1]
                        if spl.startswith('.'):
                            LTS = True
                            break
                        else:
                            LTS = False
                supported['%.2f' % float(series.version)] = (series.status, LTS)
                print >>output, "%s\t%s" % (series.name, ppa.name)
                for k, v in mintmap.iteritems():
                    if series.name in v:
                        print >>output, "%s\t%s" % (k, ppa.name)
                if not results.has_key(float(series.version)):
                    results[float(series.version)] = []
                results[float(series.version)].append(ppa.displayname)
    return results

def scrub_results(RES):
    ## This is used to scrub the old unsupported releases for the image file

    keys = RES.keys()
    keys.sort(reverse=True)

    for release in keys:
        if len(SUPLTS) < 2:
            rel = '%.2f' % release
            if supported[rel][1] == True:
                SUPLTS.append(str(release))
        else:
            rel = '%.2f' % release
            if supported[rel][1] == True:
                IGNORELTS.append(str(release))

    CLEANED = []

    hitsupport = False  # hitsupport is used when generating the graphic to include all ubuntu releases after the earliest supported one
    for key in sorted(RES.iterkeys()):
        release = '%.2f' % key
        if str(release) in IGNORELTS:
            print "Ignoring LTS release: %s" % str(release)
            pass
        elif supported[str(release)][0] == 'Supported' or supported[str(release)][0] == 'Active Development' or supported[str(release)][0] == 'Current Stable Release':
            print "Adding release: %s" % str(release)
            CLEANED.append(str(release))
            #hitsupport = True  ## Uncomment this if you want to include unsupported ubuntu versions after the earliest supported one
        elif hitsupport == True:
            CLEANED.append(str(release))
        else:
            print "Not including release: %s" % str(release)

    return CLEANED

class MythTVChart:
    def __init__(self, height_factor):
        self.release=0
        self.position=[]
        self.testNames=[]
        self.xlabels = map(lambda x: x.displayname, collection)
        fig = plt.figure(figsize=(7.5, height_factor/float(1.5)))
        self.ax1 = Subplot(fig, 111)
        fig.add_subplot(self.ax1)

    def addBuild(self, ShippingRelease, LastRelease, NumReleases, Label, Color):
        start=self.xlabels.index(ShippingRelease)
        kwargs = {}
        if LEGEND_COLORS.has_key(Color):
            kwargs['label'] = LEGEND_COLORS[Color]
            LEGEND_COLORS.pop(Color)
        bar = self.ax1.barh(self.release, NumReleases+.5, left=start, color=Color, **kwargs)
        self.testNames.append(Label)
        self.position.append(self.release+0.5)
        self.release=self.release+1

    def generatePNG(self):
        ## Axis Labels
        pylab.xticks(range(len(self.xlabels)), self.xlabels)
        pylab.xlabel("MythTV Release")
        pylab.yticks(self.position, self.testNames)
        pylab.ylabel("Ubuntu Release")

        height = 0.8
        padding = 0.2

        CURRENT, DEVELOPMENT = second_biggest_ppa()
        pylab.title("Current: %s    Development: %s    Updated: %s" % (CURRENT, DEVELOPMENT, datetime.date.today().strftime('%Y-%m-%d')))

        ## Fix chart spacing
        self.ax1.axis('tight')
        self.ax1.axis['right'].set_visible(False)
        self.ax1.axis['top'].set_visible(False)
        limits = self.ax1.axis()

        xpadding=0.1
        ypadding=0.2

        pylab.xlim(limits[0]-xpadding, limits[1]+xpadding)
        pylab.ylim(limits[2]-ypadding, limits[3]+ypadding)

        pylab.legend()

        ## Save the chart to a file
        plt.savefig(os.path.join('work-mythtv','repos-support-list.png'))

    def add_results(self, results):
        keys = results.keys()
        keys.sort(reverse=True)

        for result in keys:
            res = '%.2f' % result
            if str(res) in imageresults:
                smallest = str('1.0')
                biggest = str('0.0')
                for value in results[result]:
                    if value < smallest:
                        smallest = value
                    if value > biggest:
                        biggest = value

                if supported[str(res)][0] == 'Active Development':
                    color = '#AD7E97'
                elif supported[str(res)][0] == 'Obsolete':
                    color = 'black'
                elif supported[str(res)][1] == True and not str(res) in SUPLTS:
                    color = 'black'
                elif str(res) in SUPLTS:
                    color = '#772953'
                elif not supported[str(res)][0] == 'Obsolete':
                    color = '#E95420'
                self.addBuild(smallest, biggest, len(results[result])-1, res, color)

    def transferSupportFiles(self):
        #ssh = paramiko.SSHClient()
        #ssh.load_host_keys(os.path.expanduser(os.path.join("~", ".ssh", "known_hosts")))
        #ssh.connect('monty.mythbuntu.org', username='buildsvc')
        #sftp = ssh.open_sftp()
        #sftp.put('work-mythtv/repos-support-list.png', '/home/thomas/repos-support-list.png')
        #sftp.put('work-mythtv/repos.db', '/home/thomas/repos.db')
        #sftp.close()
        #ssh.close()

        f = open('/root/private_build_password')
        private_pass = f.read()
        f.close()

        srv = pysftp.Connection(host="people.ubuntu.com", username="tgm4883", private_key="/root/private_build_key", private_key_pass=private_pass.strip())

        # To upload the file, simple replace get with put.
        srv.chdir("public_html")
        srv.put('work-mythtv/repos-support-list.png')
        srv.put('work-mythtv/repos.db')
        srv.put('../daily_mythtv.txt')
        srv.put('../daily_mythtv_old.txt')
        srv.put('../build_mover.log')

        # Closes the connection
        srv.close()

    def moveSupportFiles(self):
        shutil.copyfile('work-mythtv/repos-support-list.png', '/root/repos-support-list.png')
        shutil.copyfile('work-mythtv/repos.db', '/root/repos.db')

output = open(os.path.join('work-mythtv','repos.db'), 'w')
load_mint()
valid_ppas()
write_header()
results = iterate_ppas()
output.close()
imageresults = scrub_results(results)
chart = MythTVChart(len(imageresults))
chart.add_results(results)
chart.generatePNG()
chart.transferSupportFiles()
##chart.moveSupportFiles()
